/*
 * Copyright (C) 2018 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of pinsis
 *
 * pinsis is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
*/

import { Injectable } from '@angular/core';
import { HttpClient, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';

export class ReportElement {
  date: Date;
  uses: number;
  fk_agent: number;
  is_sus: boolean;
  status: string;
}

export class Report {
  public sus: Array<ReportElement>;
  public not_sus: Array<ReportElement>;

}

export class graphSeries {
  public start_date: Date;
  public sus_arr: Array<number>;
  public sus_percentage_arr: Array<number>;
  public sus_total: number;

  public non_sus_arr: Array<number>;
  public non_sus_percentage_arr: Array<number>;
  public non_sus_total: number;

  public name : string;
  constructor(response: any = undefined, name: string = "") {
    this.name = name;
    
    if (typeof (response) == 'undefined') {
      this.sus_arr = [];
      this.non_sus_arr = [];
      this.start_date = new Date();
    } else {
      this.start_date = new Date(response.sus[0].date);
      this.sus_arr = response.sus.map(a => a.uses);
      this.non_sus_arr = response.not_sus.map(a => a.uses);
    }

    this.start_date.setDate(this.start_date.getDate() + 1);
    this.reloadPercentage();

  }

  reloadPercentage(total_array: Array<number> = null){
      this.sus_percentage_arr = [];
      this.sus_total = this.sus_arr.reduce((a, b) => a + b, 0);

      this.non_sus_percentage_arr = [];
      this.non_sus_total = this.non_sus_arr.reduce((a, b) => a + b, 0);

      for (let iter in this.sus_arr) {
        let total : number;
        if(total_array == null)
          total = this.sus_arr[iter] + this.non_sus_arr[iter];
        else
          total = total_array[iter];
        
        if (total == 0) {
          this.sus_percentage_arr[iter] = 0;
          this.non_sus_percentage_arr[iter] = 0;
        }
        else {
          this.sus_percentage_arr[iter] = (this.sus_arr[iter] / total);
          this.non_sus_percentage_arr[iter] = (this.non_sus_arr[iter] / total);
        }

      }
  }

}


@Injectable()
export class ReportService {
  getReports(granularity: string, status: string, startDate: Date, endDate: Date): Observable<Report> { return new Observable<Report>() };
}

@Injectable()
export class MockReportService implements ReportService {
  apiRoot: string = 'https://pinsisdev.c3sl.ufpr.br/api/data/';
  apiPeriod: string;

  constructor(private http: HttpClient) { }

  getReports(granularity: string, status: string, startDate: Date, endDate: Date): Observable<Report> {

    switch (granularity) {
      case 'Anual':
      case 'Mensal':
        this.apiPeriod = 'agentusesmonth';
        break;
      case 'Diário':
        this.apiPeriod = 'agentusesday';
        break;
    }

    startDate = new Date(startDate);
    endDate = new Date(endDate);

    let params = new HttpParams();
    params = params.append('d1', startDate.getDate().toString());
    params = params.append('m1', (startDate.getMonth() + 1).toString());
    params = params.append('y1', startDate.getFullYear().toString());
    params = params.append('d2', endDate.getDate().toString());
    params = params.append('m2', (endDate.getMonth() + 1).toString());
    params = params.append('y2', endDate.getFullYear().toString());
    params = params.append(status, "true");
    return this.http.get<Report>(this.apiRoot + this.apiPeriod, { params });
  }

}

@Injectable()
export class WebserviceReportService implements ReportService {
  apiRoot: string = 'https://pinsis.c3sl.ufpr.br/api/data/';
  apiPeriod: string;

  constructor(private http: HttpClient) { }

  getReports(granularity: string, status: string, startDate: Date, endDate: Date): Observable<Report> {

    switch (granularity) {
      case 'Anual':
      case 'Mensal':
        this.apiPeriod = 'agentusesmonth';
        break;
      case 'Diário':
        this.apiPeriod = 'agentusesday';
        break;
    }

    startDate = new Date(startDate);
    endDate = new Date(endDate);

    let params = new HttpParams();
    params = params.append('d1', startDate.getDate().toString());
    params = params.append('m1', (startDate.getMonth() + 1).toString());
    params = params.append('y1', startDate.getFullYear().toString());
    params = params.append('d2', endDate.getDate().toString());
    params = params.append('m2', (endDate.getMonth() + 1).toString());
    params = params.append('y2', endDate.getFullYear().toString());
    params = params.append(status, "true");

    return this.http.get<Report>(this.apiRoot + this.apiPeriod, { params });
  }

}
